<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="KDevCvs" version="2" translationDomain="kdevcvs">

  <Menu name="cvs_popup">
    <text context="@title:menu">CVS</text>
    <Action name="cvs_import" />
    <Action name="cvs_checkout" />
    <Action name="cvs_status" />
  </Menu>

</gui>
