/*
 * This file is part of qmljs, the QML/JS language support plugin for KDevelop
 * Copyright (c) 2014 Denis Steckelmacher <steckdenis@yahoo.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License or (at your option) version 3 or any later version
 * accepted by the membership of KDE e.V. (or its successor approved
 * by the membership of KDE e.V.), which shall act as a proxy
 * defined in Section 14 of version 3 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
    
exports.normalize = function (p) { return ""; };
exports.normalize("");

exports.join = function (path1, path2) { return ""; };
exports.join("", "");

exports.resolve = function (path) { return ""; };
exports.resolve("");

exports.relative = function (from, to) { return ""; };
exports.relative("", "");

exports.dirname = function (p) { return ""; };
exports.dirname("");

exports.basename = function (p, ext) { return ""; };
exports.basename("", "");

exports.extname = function (p) { return ""; };
exports.extname("");

exports.sep = "";

exports.delimiter = "";

